@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 1985-2009, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -38,12 +38,15 @@
 	    http_location/2,		% ?Parts, ?Location
 	    www_form_encode/2,		% Value <-> Encoded
 	    parse_url_search/2,		% Form-data <-> Form fields
-	    
+
 	    url_iri/2,			% ?URL, ?IRI
 
-	    file_name_to_url/2		% ?FileName, ?URL
+	    file_name_to_url/2,		% ?FileName, ?URL
+
+	    set_url_encoding/2		% ?Old, +New
 	  ]).
 :- use_module(library(lists)).
+:- use_module(library(error)).
 :- use_module(library(utf8)).
 
 /** <module> Analysing and constructing URL
@@ -61,6 +64,8 @@ of the standard encountered in practical use.
 
 @author	Jan Wielemaker
 @author Lukas Faulstich
+@deprecated New code should use library(uri), provided by the =clib=
+ 	    package.
 */
 
 		 /*******************************
@@ -68,9 +73,9 @@ of the standard encountered in practical use.
 		 *******************************/
 
 %%	global_url(+URL, +Base, -Global) is det.
-%	
+%
 %	Translate a possibly relative URL  into   an  absolute  one.
-%	
+%
 %	@error syntax_error(illegal_url) if URL is not legal.
 
 global_url(URL, BaseURL, Global) :-
@@ -80,7 +85,7 @@ global_url(URL, BaseURL, Global) :-
 	;   sub_atom(URL, 0, _, _, '//')
 	->  parse_url(BaseURL, [], Attributes),
 	    memberchk(protocol(Proto), Attributes),
-	    concat_atom([Proto, :, URL], Global)
+	    atomic_list_concat([Proto, :, URL], Global)
 	;   sub_atom(URL, 0, _, _, #)
 	->  (   sub_atom(BaseURL, _, _, 0, #)
 	    ->	sub_atom(URL, 1, _, 0, NoHash),
@@ -94,7 +99,7 @@ global_url(URL, BaseURL, Global) :-
 	).
 
 %%	is_absolute_url(+URL)
-%	
+%
 %	True if URL is an absolute URL. That  is, a URL that starts with
 %	a protocol identifier.
 
@@ -105,7 +110,7 @@ is_absolute_url(URL) :-
 is_absolute_url(URL) :-
 	sub_atom(URL, 0, _, _, 'ftp://'), !.
 is_absolute_url(URL) :-
-	sub_atom(URL, 0, _, _, 'file:'), !.
+	sub_atom(URL, 0, _, _, 'file://'), !.
 is_absolute_url(URL) :-
 	atom_codes(URL, Codes),
 	phrase(absolute_url, Codes, _), !.
@@ -121,12 +126,12 @@ is_absolute_url(URL) :-
 %	parse_url/2, but only deals with the   location  part of an HTTP
 %	URL. That is, the path, search   and fragment specifiers. In the
 %	HTTP protocol, the first line of a message is
-%	
+%
 %	    ==
 %	    <Action> <Location> HTTP/<version>
 %	    ==
-%	    
-%	@param Location	Atom or list of character codes.    
+%
+%	@param Location	Atom or list of character codes.
 
 http_location(Parts, Location) :-	% Parts --> Location
 	nonvar(Parts), !,
@@ -171,7 +176,7 @@ curi(A) -->
 cpath(A) -->
 	(   { memberchk(path(Path), A) }
 	->  { atom_codes(Path, Codes) },
-	    www_encode(Codes, "/+:")
+	    www_encode(Codes, "/+:,")
 	;   ""
 	).
 
@@ -205,35 +210,37 @@ catomic(A, In, Out) :-
 
 %%	csearch(+Attributes)//
 
-csearch(A)--> 
+csearch(A)-->
 	(   { memberchk(search(Parameters), A) }
 	->  csearch(Parameters, "?")
 	;   []
 	).
 
-csearch([], _) --> 
+csearch([], _) -->
 	[].
-csearch([Parameter|Parameters], Sep) --> !, 
-	Sep, 
-	cparam(Parameter), 
-	csearch(Parameters, "&"). 
+csearch([Parameter|Parameters], Sep) --> !,
+	codes(Sep),
+	cparam(Parameter),
+	csearch(Parameters, "&").
 
-cparam(Name=Value) --> !, 
-	cname(Name), 
-	"=", 
-	cvalue(Value). 
+cparam(Name=Value) --> !,
+	cname(Name),
+	"=",
+	cvalue(Value).
 cparam(NameValue) -->			% allow to feed Name(Value)
 	{ compound(NameValue), !,
 	  NameValue =.. [Name,Value]
 	},
-	cname(Name), 
-	"=", 
-	cvalue(Value). 
-cparam(Name)--> 
-	cname(Name). 
-	
+	cname(Name),
+	"=",
+	cvalue(Value).
+cparam(Name)-->
+	cname(Name).
 
-cname(Atom) --> 
+codes([]) --> [].
+codes([H|T]) --> [H], codes(T).
+
+cname(Atom) -->
 	{ atom_codes(Atom, Codes) },
 	www_encode(Codes, "").
 
@@ -248,6 +255,8 @@ cvalue(Value) -->
 	},
 	www_encode(Codes, "").
 cvalue(Codes) -->
+	{ must_be(codes, Codes)
+	},
 	www_encode(Codes, "").
 
 
@@ -261,19 +270,18 @@ cfragment(A) -->
 	www_encode(Codes, "").
 cfragment(_) -->
 	"".
-	
-		
+
+
 		 /*******************************
 		 *	      PARSING		*
 		 *******************************/
 
 %%	parse_url(+URL, -Attributes) is det.
-%%	parse_url(+URL, +BaseURL, -Attributes) is det.
 %
 %	Construct or analyse a URL. URL is an   atom  holding a URL or a
 %	variable. Parts is a list of   components.  Each component is of
 %	the format Name(Value). Defined components are:
-%	
+%
 %	    * protocol(Protocol)
 %	    The used protocol. This is, after  the optional =|url:|=, an
 %	    identifier separated from the remainder of  the URL using :.
@@ -281,22 +289,22 @@ cfragment(_) -->
 %	    specified and the URL can be parsed  as a valid HTTP url. In
 %	    addition to the RFC-1738  specified   protocols,  the =file=
 %	    protocol is supported as well.
-%	    
+%
 %	    * host(Host)
 %	    Host-name or IP-address on which   the  resource is located.
 %	    Supported by all network-based protocols.
-%	    
+%
 %	    * port(Port)
 %	    Integer port-number to access on   the \arg{Host}. This only
 %	    appears if the port is  explicitly   specified  in  the URL.
 %	    Implicit default ports (e.g.  80   for  HTTP)  do \emph{not}
 %	    appear in the part-list.
-%	    
+%
 %	    * path(Path)
 %	    (File-) path addressed by the URL. This is supported for the
 %	    =ftp=, =http= and =file= protocols. If  no path appears, the
 %	    library generates the path =|/|=.
-%	    
+%
 %	    * search(ListOfNameValue)
 %	    Search-specification of HTTP URL. This is the part after the
 %	    =|?|=, normally used to transfer data   from HTML forms that
@@ -304,16 +312,16 @@ cfragment(_) -->
 %	    www-form-encoded list of Name=Value pairs. This is mapped to
 %	    a list of Prolog Name=Value  terms   with  decoded names and
 %	    values.
-%	    
+%
 %	    * fragment(Fragment)
 %	    Fragment specification of HTTP URL. This   is the part after
 %	    the =|#|= character.
-%	    
+%
 %	The example below illustrates the all this for an HTTP URL.
-%	
+%
 %	    ==
 %	    ?- parse_url('http://swi.psy.uva.nl/message.cgi?msg=Hello+World%21#x', P).
-%	    
+%
 %	    P = [ protocol(http),
 %		  host('swi.psy.uva.nl'),
 %		  fragment(x),
@@ -322,7 +330,7 @@ cfragment(_) -->
 %		  path('/message.cgi')
 %	        ]
 %	    ==
-%	    
+%
 %	By instantiating the parts-list this predicate   can  be used to
 %	create a URL.
 
@@ -334,8 +342,13 @@ parse_url(URL, Attributes) :-
 	phrase(curl(Attributes), Codes), !,
 	atom_codes(URL, Codes).
 
+%%	parse_url(+URL, +BaseURL, -Attributes) is det.
+%
+%	Similar to parse_url/2 for relative URLs.  If URL is relative,
+%	it is resolved using the absolute URL BaseURL.
+
 parse_url(URL, BaseURL, Attributes) :-
-	nonvar(URL), !, 
+	nonvar(URL), !,
 	atom_codes(URL, Codes),
 	(   phrase(absolute_url, Codes, _)
 	->  phrase(url(Attributes), Codes)
@@ -352,19 +365,19 @@ parse_url(URL, BaseURL, Attributes) :-
 	    append(BaseA3, [path(Path)|URIA1], Attributes)
 	).
 parse_url(URL, BaseURL, Attributes) :-
-	parse_url(BaseURL, BaseAttributes), 
-	memberchk(path(BasePath), BaseAttributes), 
+	parse_url(BaseURL, BaseAttributes),
+	memberchk(path(BasePath), BaseAttributes),
 	(   memberchk(path(LocalPath), Attributes)
 	->  globalise_path(LocalPath, BasePath, Path)
 	;   Path = BasePath
-	), 	 
-	append([path(Path)|Attributes], BaseAttributes, GlobalAttributes), 
-	phrase(curl(GlobalAttributes), Chars), 
+	),
+	append([path(Path)|Attributes], BaseAttributes, GlobalAttributes),
+	phrase(curl(GlobalAttributes), Chars),
 	atom_codes(URL, Chars).
 
-	
+
 %%	globalise_path(+LocalPath, +RelativeTo, -FullPath) is det.
-%	
+%
 %	The first clause deals with the  standard URL /... global paths.
 %	The second with file://drive:path on MS-Windows.   This is a bit
 %	of a cludge, but unfortunately common practice is -especially on
@@ -392,7 +405,7 @@ make_path(Dir, Local, Path) :-
 make_path(/, Local, Path) :- !,
 	atom_concat(/, Local, Path).
 make_path(Dir, Local, Path) :-
-	concat_atom([Dir, /, Local], Path).
+	atomic_list_concat([Dir, /, Local], Path).
 
 
 %%	absolute_url//
@@ -416,7 +429,7 @@ digits(L) -->
 	digits(L, []).
 
 digits([C|T0], T) -->
-	digit(C), !, 
+	digit(C), !,
 	digits(T0, T).
 digits(T, T) -->
 	[].
@@ -461,7 +474,7 @@ http_location([path(Path)|P2]) -->
 	fragment(P3, []).
 
 %%	schema(-Atom)//
-%	
+%
 %	Schema  is  case-insensitive  and  the    canonical  version  is
 %	lowercase.
 %
@@ -479,7 +492,7 @@ schema_chars([H|T]) -->
 	schema_chars(T).
 schema_chars([]) -->
 	[].
-	
+
 schema_char(H) -->
 	[C],
 	{ C < 128,
@@ -527,7 +540,7 @@ authority(Parts, Tail) -->
 authority([host(Host)|T0], Tail) -->
 	host(Host),
 	port(T0, Tail).
-	
+
 user_info_chars([H|T]) -->
 	user_info_char(H), !,
 	user_info_chars(T).
@@ -536,7 +549,7 @@ user_info_chars([]) -->
 
 user_info_char(_) --> "@", !, {fail}.
 user_info_char(C) --> pchar(C).
-	
+
 %host(Host) --> ip_literal(Host), !.		% TBD: IP6 addresses
 host(Host) --> ip4_address(Host), !.
 host(Host) --> reg_name(Host).
@@ -647,49 +660,70 @@ segment_nz_nc_char(C) --> pchar(C).
 
 
 %%	query(-Parts, ?Tail)// is det.
-%	
+%
 %	Extract &Name=Value, ...
 
 query([search(Params)|T], T) -->
-	"?",
+	"?", !,
 	search(Params).
 query(T,T) -->
 	[].
 
-search([Parameter|Parameters])--> 
-	parameter(Parameter), !,  
-	(   "&"
+search([Parameter|Parameters])-->
+	parameter(Parameter), !,
+	(   search_sep
         ->  search(Parameters)
         ;   { Parameters = [] }
-        ). 
-search([]) --> 
+        ).
+search([]) -->
 	[].
 
-parameter(Param)--> !, 
-	parameter_component(Name), 
+parameter(Param)--> !,
+	search_chars(NameS),
+	{ atom_codes(Name, NameS)
+	},
 	(   "="
-        ->  parameter_component(Value), 
-	    { Param = (Name = Value) }
-        ;   { Param = Name }
+        ->  search_value_chars(ValueS),
+	    { atom_codes(Value, ValueS),
+	      Param = (Name = Value)
+	    }
+        ;   { Param = Name
+	    }
         ).
 
-parameter_component(Component)-->
-	search_chars(String), 
-	{ atom_codes(Component, String) }. 
-
 search_chars([C|T]) -->
-	search_char(C), !, 
+	search_char(C), !,
 	search_chars(T).
 search_chars([]) -->
 	[].
 
-search_char(_) --> "&", !, { fail }.
+search_char(_) --> search_sep, !, { fail }.
 search_char(_) --> "=", !, { fail }.
 search_char(C) --> fragment_char(C).
 
+search_value_chars([C|T]) -->
+	search_value_char(C), !,
+	search_value_chars(T).
+search_value_chars([]) -->
+	[].
+
+search_value_char(_) --> search_sep, !, { fail }.
+search_value_char(C) --> fragment_char(C).
+
+%%	search_sep// is semidet.
+%
+%	Matches a search-parameter separator.  Traditonally, this is the
+%	&-char, but these days there are `newstyle' ;-char separators.
+%
+%	@see http://perldoc.perl.org/CGI.html
+%	@tbd This should be configurable
+
+search_sep --> "&", !.
+search_sep --> ";".
+
 
 %%	fragment(-Fragment, ?Tail)//
-%	
+%
 %	Extract the fragment (after the =#=)
 
 fragment([fragment(Fragment)|T], T) -->
@@ -707,12 +741,14 @@ fragment_chars([]) -->
 
 
 %%	fragment_char(-Char)
-%	
+%
 %	Find a fragment character.
 
 fragment_char(C)   --> pchar(C), !.
 fragment_char(0'/) --> "/", !.
 fragment_char(0'?) --> "?", !.
+fragment_char(0'[) --> "[", !.		% Not according RDF3986!
+fragment_char(0']) --> "]", !.
 
 
 		 /*******************************
@@ -720,9 +756,9 @@ fragment_char(0'?) --> "?", !.
 		 *******************************/
 
 %%	pchar(-Code)//
-%	
+%
 %	unreserved|pct_encoded|sub_delim|":"|"@"
-%	
+%
 %	Performs UTF-8 decoding of percent encoded strings.
 
 pchar(0' ) --> "+", !.			%' ?
@@ -737,7 +773,7 @@ pchar(C) -->
 	percent_coded(C).
 
 %%	lwalpha(-C)//
-%	
+%
 %	Demand alpha, return as lowercase
 
 lwalpha(H) -->
@@ -759,7 +795,7 @@ drive_letter(C) -->
 		 *******************************/
 
 %%	sub_delim(?Code)
-%	
+%
 %	Sub-delimiters
 
 sub_delim(0'!).
@@ -776,7 +812,7 @@ sub_delim(0'=).
 
 
 %%	unreserved(+C)
-%	
+%
 %	Characters that can be represented without procent escaping
 %	RFC 3986, section 2.3
 
@@ -846,8 +882,12 @@ percent_encode(C, _) -->
 	{ C =< 128 }, !,
 	percent_byte(C).
 percent_encode(C, _) -->		% Unicode characters
-	{ phrase(utf8_codes([C]), Bytes) },
+	{ current_prolog_flag(url_encoding, utf8), !,
+	  phrase(utf8_codes([C]), Bytes)
+	},
 	percent_bytes(Bytes).
+percent_encode(C, _) -->
+	percent_byte(C).
 
 percent_bytes([]) -->
 	"".
@@ -905,6 +945,24 @@ utf8_cont([]) -->
 	[].
 
 
+%%	set_url_encoding(?Old, +New) is semidet.
+%
+%	Query and set the encoding for URLs.  The default is =utf8=.
+%	The only other defined value is =iso_latin_1=.
+%
+%	@tbd	Having a global flag is highly inconvenient, but a
+%		work-around for old sites using ISO Latin 1 encoding.
+
+:- set_prolog_flag(url_encoding, utf8).
+
+set_url_encoding(Old, New) :-
+	current_prolog_flag(url_encoding, Old),
+	(   Old == New
+	->  true
+	;   must_be(oneof([utf8, iso_latin_1]), New),
+	    set_prolog_flag(url_encoding, New)
+	).
+
 
 		 /*******************************
 		 *	 IRI PROCESSING		*
@@ -912,7 +970,7 @@ utf8_cont([]) -->
 
 %%	url_iri(+Encoded, -Decoded) is det.
 %%	url_iri(-Encoded, +Decoded) is det.
-%	
+%
 %	Convert between a URL, encoding in US-ASCII   and an IRI. An IRI
 %	is a fully expanded Unicode string.   Unicode  strings are first
 %	encoded into UTF-8, after which %-encoding takes place.
@@ -961,8 +1019,9 @@ parse_url_search(Codes, Fields) :-
 	is_list(Codes), !,
 	phrase(search(Fields), Codes).
 parse_url_search(Codes, Fields) :-
+	must_be(list, Fields),
 	phrase(csearch(Fields, ""), Codes).
-	
+
 
 		 /*******************************
 		 *	    FILE URLs		*
@@ -972,7 +1031,7 @@ parse_url_search(Codes, Fields) :-
 %%	file_name_to_url(-File, +URL) is semidet.
 %
 %	Translate between a filename and a file:// URL.
-%	
+%
 %	@tbd	Current implementation does not deal with paths that
 %		need special encoding.
 
