@@ -127,6 +127,7 @@ leave the details to this function.
 #include <malloc.h>
 #endif
 
+
 #ifndef __WINDOWS__
 #define closesocket(n) close((n))	/* same on Unix */
 #endif
@@ -171,21 +172,12 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 #define INITLOCK()
 #endif
 
-#define SOCK_INSTREAM	0x01
-#define SOCK_OUTSTREAM	0x02
-#define SOCK_BIND	0x04		/* What have we done? */
-#define SOCK_LISTEN	0x08
-#define SOCK_CONNECT	0x10		/* Connected (client) socket */
-#define SOCK_ACCEPT	0x20		/* Accepted (server) socket */
-#define SOCK_NONBLOCK	0x40		/* Set to non-blocking mode */
-#define SOCK_DISPATCH   0x80		/* do not dispatch events */
-
 #define set(s, f)   ((s)->flags |= (f))
 #define clear(s, f) ((s)->flags &= ~(f))
 #define true(s, f)  ((s)->flags & (f))
 #define false(s, f) (!true(s, f))
 
-#define SOCK_MAGIC 0x38da3f2c
+#define PLSOCK_MAGIC 0x38da3f2c
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 NOTE: We must lock  the  structure   to  avoid  freeSocket() called from
@@ -194,7 +186,7 @@ that are concurrently executed in the socket thread.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 typedef struct _plsocket
-{ int		    magic;		/* SOCK_MAGIC */
+{ int		    magic;		/* PLSOCK_MAGIC */
   nbio_sock_t	    id;			/* Integer id */
   SOCKET	    socket;		/* The OS socket */
   int		    flags;		/* Misc flags */
@@ -248,7 +240,6 @@ typedef struct _plsocket
 } plsocket;
 
 static plsocket *allocSocket(SOCKET socket);
-static plsocket *nbio_to_plsocket(nbio_sock_t socket);
 #ifdef __WINDOWS__
 static plsocket *lookupOSSocket(SOCKET socket);
 static const char *WinSockError(unsigned long eno);
@@ -357,7 +348,7 @@ nbio_fcntl(nbio_sock_t socket, int op, int arg)
 	  non_block = 1;
 	  rval = ioctlsocket(s->socket, FIONBIO, &non_block);
 	  if ( rval )
-	  { s->flags |= SOCK_NONBLOCK;
+	  { s->flags |= PLSOCK_NONBLOCK;
 	    return 0;
 	  }
 
@@ -405,7 +396,7 @@ doneRequest(plsocket *s)
 
 static int
 waitRequest(plsocket *s)
-{ assert(s->magic == SOCK_MAGIC);
+{ assert(s->magic == PLSOCK_MAGIC);
 
   DEBUG(2, Sdprintf("[%d] (%ld): Waiting for %s on %d ...",
 		    PL_thread_self(), s->thread,
@@ -413,7 +404,7 @@ waitRequest(plsocket *s)
 
   for(;;)
   { MSG msg;
-  
+
     if ( PL_handle_signals() < 0 )
     { DEBUG(1, Sdprintf("[%d]: Exception\n", PL_thread_self()));
       return FALSE;
@@ -423,7 +414,7 @@ waitRequest(plsocket *s)
       return TRUE;
     }
 
-    if ( false(s, SOCK_DISPATCH) )
+    if ( false(s, PLSOCK_DISPATCH) )
     { if ( !GetMessage(&msg, NULL, WM_DONE, WM_DONE) )
 	return FALSE;
     } else if ( GetMessage(&msg, NULL, 0, 0) )
@@ -444,7 +435,7 @@ nbio_wait(nbio_sock_t socket, nbio_request request)
   if ( !(s=nbio_to_plsocket(socket)) )
     return -1;
 
-  s->flags  |= SOCK_WAITING;
+  s->flags  |= PLSOCK_WAITING;
   s->done    = FALSE;
   s->error   = 0;
   s->thread  = GetCurrentThreadId();
@@ -457,7 +448,7 @@ nbio_wait(nbio_sock_t socket, nbio_request request)
 
   for(;;)
   { MSG msg;
-  
+
     if ( PL_handle_signals() < 0 )
     { DEBUG(1, Sdprintf("[%d]: Exception\n", PL_thread_self()));
       return -1;
@@ -467,7 +458,7 @@ nbio_wait(nbio_sock_t socket, nbio_request request)
       return 0;
     }
 
-    if ( false(s, SOCK_DISPATCH) )
+    if ( false(s, PLSOCK_DISPATCH) )
     { if ( !GetMessage(&msg, NULL, WM_DONE, WM_DONE) )
 	return FALSE;
     } else if ( GetMessage(&msg, NULL, 0, 0) )
@@ -482,7 +473,7 @@ nbio_wait(nbio_sock_t socket, nbio_request request)
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-nbio_select() selects using a set of socket streams. 
+nbio_select() selects using a set of socket streams.
 
 NOTE: The Windows versions uses our   nbio_sock_t abstraction, while the
 other version uses  the  raw  Unix   file  descriptors  referencing  the
@@ -510,11 +501,11 @@ nbio_select(int n,
       { plsocket *s = nbio_to_plsocket(i);
 
 	if ( s )
-	{ s->flags  |= SOCK_WAITING;
+	{ s->flags  |= PLSOCK_WAITING;
 	  s->done    = FALSE;
 	  s->error   = 0;
 	  s->thread  = GetCurrentThreadId();
-	  s->request = (s->flags & SOCK_LISTEN) ? REQ_ACCEPT : REQ_READ;
+	  s->request = (s->flags & PLSOCK_LISTEN) ? REQ_ACCEPT : REQ_READ;
 	  sockets[i] = s;
 	} else
 	{ DEBUG(2, Sdprintf("nbio_select(): no socket for %d\n", i));
@@ -545,7 +536,7 @@ nbio_select(int n,
     { DEBUG(1, Sdprintf("[%d]: Exception\n", PL_thread_self()));
       return -1;
     }
-    
+
     for(ready=0, i=0, s=sockets; i<n; i++, s++)
     { if ( *s && (*s)->done )
       { ready++;
@@ -592,26 +583,26 @@ nbio_select(int n,
 
 static int
 placeRequest(plsocket *s, nbio_request request)
-{ if ( s->magic != SOCK_MAGIC )
+{ if ( s->magic != PLSOCK_MAGIC )
     Sdprintf("placeRequest: %p has bad magic\n", s);
 
   s->error   = 0;
   s->done    = FALSE;
   s->thread  = GetCurrentThreadId();
   s->request = request;
-  clear(s, SOCK_WAITING);
+  clear(s, PLSOCK_WAITING);
 
   SendMessage(State()->hwnd, WM_REQUEST, 1, (LPARAM)&s);
   DEBUG(2, Sdprintf("%d (%ld): Placed %s request for %d\n",
 		    PL_thread_self(), s->thread,
 		    request_name(request), (int)s->socket));
 
-  return TRUE; 
+  return TRUE;
 }
 
 static int
 doRequest(plsocket *s)
-{ if ( s->magic != SOCK_MAGIC )
+{ if ( s->magic != PLSOCK_MAGIC )
     Sdprintf("doRequest: %p has bad magic\n", s);
 
   switch(s->request)
@@ -621,7 +612,7 @@ doRequest(plsocket *s)
       if ( s->w32_flags & FD_CONNECT )
       { s->w32_flags &= ~FD_CONNECT;
 
-	if ( true(s, SOCK_WAITING) )
+	if ( true(s, PLSOCK_WAITING) )
 	{ doneRequest(s);
 	  break;
 	}
@@ -630,7 +621,7 @@ doRequest(plsocket *s)
 		     (struct sockaddr*)&s->rdata.connect.addr,
 		     (int)s->rdata.connect.addrlen) )
 	{ s->error = WSAGetLastError();
-	  
+
 	  switch(s->error)
 	  { case WSAEWOULDBLOCK:
 	    case WSAEINVAL:
@@ -654,7 +645,7 @@ doRequest(plsocket *s)
       { SOCKET slave;
 
 	s->w32_flags &= ~FD_ACCEPT;
-	if ( true(s, SOCK_WAITING) )
+	if ( true(s, PLSOCK_WAITING) )
 	{ doneRequest(s);
 	  break;
 	}
@@ -665,7 +656,7 @@ doRequest(plsocket *s)
 
 	if ( slave == SOCKET_ERROR )
 	{ s->error = WSAGetLastError();
-	  
+
 	  DEBUG(2, Sdprintf("Accept(%d): %s\n",
 			    (int)s->socket, WinSockError(s->error)));
 
@@ -679,7 +670,7 @@ doRequest(plsocket *s)
 	  DEBUG(2, Sdprintf("Accept(%d) --> %d\n",
 			    (int)s->socket, (int)slave));
 	  if ( (pls = allocSocket(slave)) )
-	  { pls->flags |= SOCK_ACCEPT;	/* requests */
+	  { pls->flags |= PLSOCK_ACCEPT;	/* requests */
 
 	    s->rdata.accept.slave = pls->id;
 	    s->error = 0;
@@ -695,7 +686,7 @@ doRequest(plsocket *s)
       if ( s->w32_flags & (FD_READ|FD_CLOSE) )
       { s->w32_flags &= ~FD_READ;
 
-	if ( true(s, SOCK_WAITING) )
+	if ( true(s, PLSOCK_WAITING) )
 	{ doneRequest(s);
 	  break;
 	}
@@ -721,7 +712,7 @@ doRequest(plsocket *s)
       if ( s->w32_flags & (FD_READ|FD_CLOSE) )
       { s->w32_flags &= ~FD_READ;
 
-	if ( true(s, SOCK_WAITING) )
+	if ( true(s, PLSOCK_WAITING) )
 	{ doneRequest(s);
 	  break;
 	}
@@ -754,7 +745,7 @@ doRequest(plsocket *s)
 
 	s->w32_flags &= ~FD_WRITE;
 
-	if ( true(s, SOCK_WAITING) )
+	if ( true(s, PLSOCK_WAITING) )
 	{ doneRequest(s);
 	  break;
 	}
@@ -775,7 +766,7 @@ doRequest(plsocket *s)
 	  doneRequest(s);
 	} else
 	  s->error = 0;
-	  
+
 	s->rdata.write.written += n;
 	if ( s->rdata.write.written >= s->rdata.write.size )
 	{ s->rdata.write.bytes = (int)s->rdata.write.written;
@@ -788,7 +779,7 @@ doRequest(plsocket *s)
 
 	s->w32_flags &= ~FD_WRITE;
 
-	if ( true(s, SOCK_WAITING) )
+	if ( true(s, PLSOCK_WAITING) )
 	{ doneRequest(s);
 	  break;
 	}
@@ -811,7 +802,7 @@ doRequest(plsocket *s)
 	  doneRequest(s);
 	} else
 	  s->error = 0;
-	  
+
 	s->rdata.sendto.bytes = n;
 	doneRequest(s);
       }
@@ -830,7 +821,7 @@ socket_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
     for(i=0; i<n; i++)
     { if ( s[i] )
       { __try
-	{ if ( s[i]->magic != SOCK_MAGIC )
+	{ if ( s[i]->magic != PLSOCK_MAGIC )
 	  { goto nosocket;
 	  }
 	} __except(EXCEPTION_EXECUTE_HANDLER)
@@ -855,14 +846,14 @@ socket_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
     if ( evt&FD_CLOSE )
     { DEBUG(1,
 	    { char *nm = event_name(evt);
-	      Sdprintf("WM_SOCKET on %d: ev=(%s); err=%s\n", 
+	      Sdprintf("WM_SOCKET on %d: ev=(%s); err=%s\n",
 		       (int)sock, nm, err ? WinSockError(err) : "none");
 	      free(nm);
 	    });
     } else
     { DEBUG(3,
 	    { char *nm = event_name(evt);
-	      Sdprintf("WM_SOCKET on %d: ev=(%s); err=%s\n", 
+	      Sdprintf("WM_SOCKET on %d: ev=(%s); err=%s\n",
 		       (int)sock, nm, err ? WinSockError(err) : "none");
 	      free(nm);
 	    });
@@ -882,7 +873,10 @@ socket_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
       }
 
       s->w32_flags |= evt;
-      if ( err )
+      if ( err == WSAECONNABORTED && s->request == REQ_READ )
+      { s->rdata.read.bytes = 0;
+	doneRequest(s);
+      } else if ( err )
       { SOCKET sock = s->socket;
 
 	s->error = err;
@@ -1035,10 +1029,10 @@ without dispatching if no input is available.
 
 static int
 wait_socket(plsocket *s)
-{ if ( true(s, SOCK_DISPATCH) )
+{ if ( true(s, PLSOCK_DISPATCH) )
   { int fd = s->socket;
 
-    if ( true(s, SOCK_NONBLOCK) && !PL_dispatch(fd, PL_DISPATCH_INSTALLED) )
+    if ( true(s, PLSOCK_NONBLOCK) && !PL_dispatch(fd, PL_DISPATCH_INSTALLED) )
     { fd_set rfds;
       struct timeval tv;
 
@@ -1046,7 +1040,7 @@ wait_socket(plsocket *s)
       FD_SET(fd, &rfds);
       tv.tv_sec = 0;
       tv.tv_usec = 250000;
-      
+
       select(fd+1, &rfds, NULL, NULL, &tv);
       return TRUE;
     } else
@@ -1081,7 +1075,7 @@ nbio_fcntl(nbio_sock_t socket, int op, int arg)
 
   if ( rc == 0 )
   { if ( op == F_SETFL && arg == O_NONBLOCK )
-      s->flags |= SOCK_NONBLOCK;
+      s->flags |= PLSOCK_NONBLOCK;
   } else
     nbio_error(errno, TCP_ERRNO);
 
@@ -1099,7 +1093,7 @@ int
 nbio_select(int n,
 	    fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
 	    struct timeval *timeout)
-{ return select(n, readfds, writefds, exceptfds, timeout); 
+{ return select(n, readfds, writefds, exceptfds, timeout);
 }
 
 #endif /*__WINDOWS__*/
@@ -1133,7 +1127,7 @@ lookupOSSocket(SOCKET socket)
   { if ( (p=sockets[i]) && p->socket == socket )
     { UNLOCK();
 
-      if ( p->magic != SOCK_MAGIC )
+      if ( p->magic != PLSOCK_MAGIC )
       { errno = EINVAL;
 	DEBUG(1, Sdprintf("Invalid OS socket: %d\n", socket));
 	return NULL;
@@ -1161,7 +1155,7 @@ nbio_to_plsocket_nolock(nbio_sock_t socket)
 
   p = sockets[socket];
 
-  if ( !p || p->magic != SOCK_MAGIC )
+  if ( !p || p->magic != PLSOCK_MAGIC )
   { DEBUG(1, Sdprintf("Invalid NBIO socket: %d\n", socket));
     errno = EINVAL;
     return NULL;
@@ -1171,6 +1165,12 @@ nbio_to_plsocket_nolock(nbio_sock_t socket)
 }
 
 
+SOCKET
+plsocket_handle(plsocket_ptr pls)
+{ return pls->socket;
+}
+
+
 static plsocket *
 nbio_to_plsocket_raw(nbio_sock_t socket)
 { plsocket *s;
@@ -1183,7 +1183,7 @@ nbio_to_plsocket_raw(nbio_sock_t socket)
 }
 
 
-static plsocket *
+plsocket *
 nbio_to_plsocket(nbio_sock_t socket)
 { plsocket *p;
 
@@ -1258,12 +1258,12 @@ allocSocket(SOCKET socket)
   UNLOCK();
 
   assert(i<socks_allocated);
-  
+
   memset(p, 0, sizeof(*p));
   p->id     = (int)i;			/* place in the array */
   p->socket = socket;
-  p->flags  = SOCK_DISPATCH;		/* by default, dispatch */
-  p->magic  = SOCK_MAGIC;
+  p->flags  = PLSOCK_DISPATCH;		/* by default, dispatch */
+  p->magic  = PLSOCK_MAGIC;
 #ifdef __WINDOWS__
   p->w32_flags = 0;
   p->request   = REQ_NONE;
@@ -1284,7 +1284,7 @@ freeSocket(plsocket *s)
   SOCKET sock;
 
   DEBUG(2, Sdprintf("Closing %d\n", s->id));
-  if ( !s || s->magic != SOCK_MAGIC )
+  if ( !s || s->magic != PLSOCK_MAGIC )
   { errno = EINVAL;
     return -1;
   }
@@ -1335,7 +1335,7 @@ WinSockError(unsigned long eno)
   static HMODULE netmsg = 0;
   static int netmsg_loaded = FALSE;
   unsigned long flags = (FORMAT_MESSAGE_FROM_SYSTEM|
-			 FORMAT_MESSAGE_IGNORE_INSERTS); 
+			 FORMAT_MESSAGE_IGNORE_INSERTS);
 
   if ( !netmsg_loaded )
   { netmsg_loaded = TRUE;
@@ -1343,7 +1343,7 @@ WinSockError(unsigned long eno)
     if ( !netmsg )
       Sdprintf("failed to load netmsg.dll\n");
     else
-      Sdprintf("Loaded netmsg.dll as %p\n", netmsg); 
+      Sdprintf("Loaded netmsg.dll as %p\n", netmsg);
   }
 
   if ( netmsg )
@@ -1535,12 +1535,12 @@ nbio_init(const char *module)
   }
   initialised = TRUE;
 
-  FUNCTOR_module2 = PL_new_functor(PL_new_atom(":"), 2);
-  FUNCTOR_ip4     = PL_new_functor(PL_new_atom("ip"), 4);
-  FUNCTOR_ip1     = PL_new_functor(PL_new_atom("ip"), 1);
-  ATOM_any	  = PL_new_atom("any");
-  ATOM_broadcast  = PL_new_atom("broadcast");
-  ATOM_loopback   = PL_new_atom("loopback");
+  FUNCTOR_module2  = PL_new_functor(PL_new_atom(":"), 2);
+  FUNCTOR_ip4	   = PL_new_functor(PL_new_atom("ip"), 4);
+  FUNCTOR_ip1	   = PL_new_functor(PL_new_atom("ip"), 1);
+  ATOM_any	   = PL_new_atom("any");
+  ATOM_broadcast   = PL_new_atom("broadcast");
+  ATOM_loopback	   = PL_new_atom("loopback");
 
 #ifdef __WINDOWS__
 { WSADATA WSAData;
@@ -1565,7 +1565,7 @@ nbio_init(const char *module)
   UNLOCK();
   return TRUE;
 }
-	
+
 
 int
 nbio_cleanup(void)
@@ -1590,7 +1590,7 @@ nbio_sock_t
 nbio_socket(int domain, int type, int protocol)
 { SOCKET sock;
   plsocket *s;
-	
+
   assert(initialised);
 
   if ( (sock = socket(domain, type , protocol)) < 0)
@@ -1621,21 +1621,21 @@ nbio_closesocket(nbio_sock_t socket)
     return -1;
   }
 
-  if ( true(s, SOCK_OUTSTREAM|SOCK_INSTREAM) )
+  if ( true(s, PLSOCK_OUTSTREAM|PLSOCK_INSTREAM) )
   { int flags = s->flags;		/* may drop out! */
 
-    if ( flags & SOCK_INSTREAM )
+    if ( flags & PLSOCK_INSTREAM )
     { assert(s->input);
       Sclose(s->input);
     }
-    if ( flags & SOCK_OUTSTREAM )
+    if ( flags & PLSOCK_OUTSTREAM )
     { assert(s->output);
       Sclose(s->output);
     }
   } else
   {
 #ifdef __WINDOWS__
-    if ( true(s, SOCK_CONNECT) )
+    if ( true(s, PLSOCK_CONNECT) )
     { if ( s->socket >= 0 )
 	shutdown(s->socket, SD_SEND);
     }
@@ -1647,7 +1647,6 @@ nbio_closesocket(nbio_sock_t socket)
   return 0;
 }
 
-
 int
 nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 { plsocket *s;
@@ -1709,11 +1708,11 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
     }
     case TCP_DISPATCH:
     { int val = va_arg(args, int);
-      
+
       if ( val )
-	set(s, SOCK_DISPATCH);
+	set(s, PLSOCK_DISPATCH);
       else
-	clear(s, SOCK_DISPATCH);
+	clear(s, PLSOCK_DISPATCH);
 
       rc = 0;
 
@@ -1722,7 +1721,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
     case TCP_INSTREAM:
     { IOSTREAM *in = va_arg(args, IOSTREAM*);
 
-      s->flags |= SOCK_INSTREAM;
+      s->flags |= PLSOCK_INSTREAM;
       s->input = in;
 
       rc = 0;
@@ -1732,7 +1731,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
     case TCP_OUTSTREAM:
     { IOSTREAM *out = va_arg(args, IOSTREAM*);
 
-      s->flags |= SOCK_OUTSTREAM;
+      s->flags |= PLSOCK_OUTSTREAM;
       s->output = out;
 
       rc = 0;
@@ -1753,12 +1752,12 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 int
 nbio_get_flags(nbio_sock_t socket)
 { plsocket *s;
-  
+
   if ( !(s = nbio_to_plsocket(socket)) )
     return -1;
 
   return s->flags;
-} 
+}
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1771,7 +1770,7 @@ nbio_get_port(term_t Port, int *port)
 
   if ( PL_get_atom_chars(Port, &name) )
   { struct servent *service;
-    
+
     if ( !(service = getservbyname(name, "tcp")) )
       return pl_error(NULL, 0, NULL, ERR_EXISTENCE, "service", Port);
 
@@ -1795,7 +1794,7 @@ or the name of a registered port (e.g. 'smtp').
 int
 nbio_get_sockaddr(term_t Address, struct sockaddr_in *addr)
 { int		  port;
-	
+
   addr->sin_family = AF_INET;
   addr->sin_addr.s_addr = INADDR_ANY;
 
@@ -1899,7 +1898,7 @@ nbio_bind(nbio_sock_t socket, struct sockaddr *my_addr, size_t addrlen)
     return -1;
   }
 
-  s->flags |= SOCK_BIND;
+  s->flags |= PLSOCK_BIND;
 
   return 0;
 }
@@ -1910,10 +1909,10 @@ nbio_connect(nbio_sock_t socket,
 	     const struct sockaddr *serv_addr,
 	     size_t addrlen)
 { plsocket *s;
-       
+
   if ( !(s = nbio_to_plsocket(socket)) )
     return -1;
-  
+
 #ifdef __WINDOWS__
   if ( connect(s->socket, serv_addr, (int)addrlen) )
   { s->error = WSAGetLastError();
@@ -1945,7 +1944,7 @@ nbio_connect(nbio_sock_t socket,
   }
 #endif
 
-  s->flags |= SOCK_CONNECT;
+  s->flags |= PLSOCK_CONNECT;
 
   return 0;
 }
@@ -1955,7 +1954,7 @@ nbio_sock_t
 nbio_accept(nbio_sock_t master, struct sockaddr *addr, socklen_t *addrlen)
 { SOCKET slave;
   plsocket *m, *s;
-	
+
   if ( !(m = nbio_to_plsocket(master)) )
     return -1;
 
@@ -2008,11 +2007,11 @@ nbio_accept(nbio_sock_t master, struct sockaddr *addr, socklen_t *addrlen)
   }
 
 #endif /*__WINDOWS__*/
-  
+
   s = allocSocket(slave);
-  s->flags |= SOCK_ACCEPT;
+  s->flags |= PLSOCK_ACCEPT;
 #ifndef __WINDOWS__
-  if ( true(s, SOCK_NONBLOCK) )
+  if ( true(s, PLSOCK_NONBLOCK) )
     nbio_setopt(slave, TCP_NONBLOCK);
 #endif
 
@@ -2032,7 +2031,7 @@ nbio_listen(nbio_sock_t socket, int backlog)
     return -1;
   }
 
-  s->flags |= SOCK_LISTEN;
+  s->flags |= PLSOCK_LISTEN;
 
   return 0;
 }
@@ -2192,9 +2191,9 @@ nbio_close_input(nbio_sock_t socket)
 
   DEBUG(2, Sdprintf("[%d]: nbio_close_input(%d, flags=0x%x)\n",
 		    PL_thread_self(), socket, s->flags));
-  s->flags &= ~SOCK_INSTREAM;
+  s->flags &= ~PLSOCK_INSTREAM;
 #ifdef __WINDOWS__
-  if ( false(s, SOCK_LISTEN) )
+  if ( false(s, PLSOCK_LISTEN) )
   { SOCKET sock;
 
     if ( (sock=s->socket) < 0 )
@@ -2211,7 +2210,7 @@ nbio_close_input(nbio_sock_t socket)
 #endif
 
   s->input = NULL;
-  if ( !(s->flags & (SOCK_INSTREAM|SOCK_OUTSTREAM)) )
+  if ( !(s->flags & (PLSOCK_INSTREAM|PLSOCK_OUTSTREAM)) )
     return freeSocket(s);
 
   return rc;
@@ -2234,7 +2233,7 @@ nbio_close_output(nbio_sock_t socket)
     SOCKET sock;
 #endif
 
-    s->flags &= ~SOCK_OUTSTREAM;
+    s->flags &= ~PLSOCK_OUTSTREAM;
 #if __WINDOWS__
     if ( (sock=s->socket) < 0 )
     { s->error = WSAECONNRESET;
@@ -2259,7 +2258,7 @@ nbio_close_output(nbio_sock_t socket)
 
   DEBUG(3, Sdprintf("%d->flags = 0x%x\n", socket, s->flags));
   s->output = NULL;
-  if ( !(s->flags & (SOCK_INSTREAM|SOCK_OUTSTREAM)) )
+  if ( !(s->flags & (PLSOCK_INSTREAM|PLSOCK_OUTSTREAM)) )
     return freeSocket(s);
 
   return rc;
@@ -2313,7 +2312,7 @@ nbio_recvfrom(int socket, void *buf, size_t bufSize, int flags,
 #else /*__WINDOWS__*/
 
   for(;;)
-  { if ( !wait_socket(s) )
+  { if ( (flags & MSG_DONTWAIT) == 0 && !wait_socket(s) )
     { errno = EPLEXCEPTION;
       return -1;
     }
@@ -2325,6 +2324,10 @@ nbio_recvfrom(int socket, void *buf, size_t bufSize, int flags,
       { errno = EPLEXCEPTION;
 	return -1;
       }
+
+      if((flags & MSG_DONTWAIT) != 0)
+        break;
+
       continue;
     }
 
@@ -2355,14 +2358,14 @@ nbio_sendto(nbio_sock_t socket, void *buf, size_t bufSize, int flags,
   n = sendto(s->socket, buf, (int)bufSize, flags, to, tolen);
   if ( n < 0 )
   { int error = WSAGetLastError();
-    
+
     if ( error == WSAEWOULDBLOCK )
       goto wouldblock;
 
     DEBUG(1, Sdprintf("[%d]: sendto(%d, %d bytes): %s\n",
 		      PL_thread_self(), s->socket, (int)bufSize,
 		      WinSockError(error)));
-    
+
     s->error = error;
     return -1;
   } else
