@@ -45,7 +45,7 @@
 
 	    udp_socket/1,		% -Socket
 	    udp_receive/4,		% +Socket, -Data, -Sender, +Options
-	    udp_send/4			% +Socket, +Data, +Sender, +Options
+	    udp_send/4  		% +Socket, +Data, +Sender, +Options
 	  ]).
 :- use_module(library(shlib)).
 
@@ -54,9 +54,7 @@ These predicates are documented in the source-distribution of the package
 `clib'.  See also the SWI-Prolog home-page at http://www.swi-prolog.org
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-:- initialization
-   load_foreign_library(foreign(socket), install_socket).
-
+:- use_foreign_library(foreign(socket), install_socket).
 
 		 /*******************************
 		 *	   COMPATIBILITY	*
