@@ -117,17 +117,17 @@ encoding.
 %	flags to ensure  with  proper   multi-threaded  behaviour  where
 %	setting an option is local to the   thread and new threads start
 %	with the options from the parent thread.  Defined options are:
-%	
+%
 %		* dialect(Dialect)
 %		One of =html= (default) or =xhtml=.
-%		
+%
 %		* doctype(+DocType)
 %		Set the =|<|DOCTYPE|= DocType =|>|= line for page//1 and
 %		page//2.
-%		
+%
 %		* content_type(+ContentType)
 %		Set the =|Content-type|= for reply_html_page/2
-%		
+%
 %	Note  that  the  doctype  is  covered    by  two  prolog  flags:
 %	=html_doctype= for the html dialect  and =xhtml_doctype= for the
 %	xhtml dialect. Dialect muct be switched before doctype.
@@ -202,9 +202,8 @@ init_options :-
 	    fail
 	;   true
 	).
-	
-:- initialization
-	init_options.
+
+:- init_options.
 
 %%	xml_header(-Header)
 %
@@ -263,9 +262,9 @@ pagehead(Head) -->
 	html(Head).
 pagehead(Head) -->
 	{ strip_module(Head, M, _),
-	  hook_module(M, head//1)
+	  hook_module(M, HM, head//1)
 	}, !,
-	M:head(Head).
+	HM:head(Head).
 pagehead(Head) -->
 	html(head(Head)).
 
@@ -276,16 +275,16 @@ pagebody(Body) -->
 	html(Body).
 pagebody(Body) -->
 	{ strip_module(Body, M, _),
-	  hook_module(M, body//1)
+	  hook_module(M, HM, body//1)
 	}, !,
-	M:body(Body).
+	HM:body(Body).
 pagebody(Body) -->
 	html(body(Body)).
 
 
-hook_module(M, PI) :-
+hook_module(M, M, PI) :-
 	current_predicate(M:PI), !.
-hook_module(user, PI) :-
+hook_module(_, user, PI) :-
 	current_predicate(user:PI).
 
 %%	html(+Content:dom)// is det
@@ -322,10 +321,10 @@ do_expand(Fmt-Args, _) --> !,
 	{ format(string(String), Fmt, Args)
 	},
 	html_quoted(String).
-do_expand(\List, _) -->
+do_expand(\List, Module) -->
 	{ is_list(List)
 	}, !,
-	List.
+	raw(List, Module).
 do_expand(\Term, Module, In, Rest) :- !,
 	call(Module:Term, In, Rest).
 do_expand(Module:Term, _) --> !,
@@ -336,7 +335,9 @@ do_expand(script(Content), _) --> !,	% general CDATA declared content elements?
 	],
 	html_end(script).
 do_expand(&(Entity), _) --> !,
-	{ format(string(String), '&~w;', [Entity])
+	{   integer(Entity)
+	->  format(string(String), '&#~d;', [Entity])
+	;   format(string(String), '&~w;', [Entity])
 	},
 	[ String ].
 do_expand(Token, _) -->
@@ -386,6 +387,29 @@ check_non_empty(_, Tag, Term) :-
 	print_message(warning, format('Using empty element with content: ~p', [Term])).
 check_non_empty(_, _, _).
 
+%%	raw(+List, +Modules)// is det.
+%
+%	Emit unquoted (raw) output used for scripts, etc.
+
+raw([], _) -->
+	[].
+raw([H|T], Module) -->
+	raw_element(H, Module),
+	raw(T, Module).
+
+raw_element(Var, _) -->
+	{ var(Var), !,
+	  instantiation_error(Var)
+	}.
+raw_element(\Term, Module, In, Rest) :- !,
+	call(Module:Term, In, Rest).
+raw_element(Fmt-Args, _) --> !,
+	{ format(string(S), Fmt, Args) },
+	[S].
+raw_element(Value, _) -->
+	{ must_be(atomic, Value) },
+	[Value].
+
 
 %%	html_begin(+Env)// is det.
 %%	html_end(+End)// is det
@@ -395,7 +419,7 @@ check_non_empty(_, _, _).
 %	exceptional  cases.  Normal  applications    use   html//1.  The
 %	following two fragments are identical, where we prefer the first
 %	as it is more concise and less error-prone.
-%	
+%
 %	==
 %		html(table(border=1, \table_content))
 %	==
@@ -454,18 +478,18 @@ xhtml_empty(Env, Attributes) -->
 %	(http://www.w3.org/2006/07/SWD/RDFa/syntax/), embedding RDF   in
 %	(x)html provides a typical  usage  scenario   where  we  want to
 %	publish the required namespaces in the header. We can define:
-%	
+%
 %	==
 %	rdf_ns(Id) -->
 %		{ rdf_global_id(Id:'', Value) },
 %		xhtml_ns(Id, Value).
 %	==
-%	
+%
 %	After which we can use rdf_ns//1 as  a normal rule in html//1 to
 %	publish namespaces from library(semweb/rdf_db).   Note that this
 %	macro only has effect if  the  dialect   is  set  to =xhtml=. In
 %	=html= mode it is silently ignored.
-%	
+%
 %	The required =xmlns= receiver  is   installed  by  html_begin//1
 %	using the =html= tag and thus is   present  in any document that
 %	opens the outer =html= environment through this library.
@@ -602,11 +626,11 @@ search_parameter(Term) -->
 %	appearing in the document  structure   is  normally quoted using
 %	these rules. I.e. the following emits  properly quoted bold text
 %	regardless of the content of Text:
-%	
+%
 %	==
 %		html(b(Text))
 %	==
-%	
+%
 %	@tbd	Assumes UTF-8 encoding of the output.
 
 html_quoted(Text) -->
@@ -619,7 +643,7 @@ html_quoted(Text) -->
 %	included in double-quotes.  Note   that  -like  html_quoted//1-,
 %	attributed   values   printed   through   html//1   are   quoted
 %	atomatically.
-%	
+%
 %	@tbd	Assumes UTF-8 encoding of the output.
 
 html_quoted_attribute(Text) -->
@@ -638,10 +662,10 @@ html_quoted_attribute(Text) -->
 %	by mainman/1 from  print_html/1   or  html_print_length/2. These
 %	commands are called in the calling   context of the html_post//2
 %	call.
-%	
+%
 %	A typical usage scenario is to  get   required  CSS links in the
 %	document head in a reusable fashion. First, we define css//1 as:
-%	
+%
 %	==
 %	css(URL) -->
 %		html_post(css,
@@ -650,10 +674,10 @@ html_quoted_attribute(Text) -->
 %				 href(URL)
 %			       ])).
 %	==
-%	
+%
 %	Next we insert the _unique_ CSS links, in the pagehead using the
 %	following call to reply_html_page/2:
-%	
+%
 %	==
 %		reply_html_page([ title(...),
 %				  \html_receive(css)
@@ -670,7 +694,7 @@ html_post(Id, Content) -->
 %	Receive posted HTML tokens. Unique   sequences  of tokens posted
 %	with  html_post//2  are  inserted   at    the   location   where
 %	html_receive//1 appears.
-%	
+%
 %	@see	The local predicate sorted_html//1 handles the output of
 %		html_receive//1.
 %	@see	html_receive//2 allows for post-processing the posted
@@ -685,7 +709,7 @@ html_receive(Id) -->
 %	called to process all messages posted to the channal at the time
 %	output is generated. Handler is a   grammar  rule that is called
 %	with three extra arguments.
-%	
+%
 %	    1. A list of Module:Term, of posted terms.  Module is the
 %	       contest module of html_post and Term is the unmodified
 %	       term.  Members are in the order posted and may contain
@@ -693,10 +717,10 @@ html_receive(Id) -->
 %	    2. DCG input list.  The final output must be produced by a
 %	       call to html//1.
 %	    3. DCG output list.
-%	    
+%
 %	Typically, Handler collects the posted   terms,  creating a term
 %	suitable for html//1 and finally calls html//1.
-    
+
 html_receive(Id, Handler) -->
 	{ strip_module(Handler, M, P) },
 	[ mailbox(Id, accept(M:P, _)) ].
@@ -745,7 +769,7 @@ mail_handlers([post(Module,HTML)|T0], H, [Module:HTML|T]) :- !,
 	mail_handlers(T0, H, T).
 mail_handlers([H|T0], [H|T], C) :-
 	mail_handlers(T0, T, C).
-	
+
 extend_args(Term, Extra, NewTerm) :-
 	Term =.. [Name|Args],
 	append(Args, [Extra], NewArgs),
@@ -755,7 +779,7 @@ extend_args(Term, Extra, NewTerm) :-
 %
 %	Default  handlers  for  html_receive//1.  It  sorts  the  posted
 %	objects to create a unique list.
-%	
+%
 %	@bug	Elements can differ just on the module.  Ideally we
 %		should phrase all members, sort the list of list of
 %		tokens and emit the result.  Can we do better?
@@ -769,7 +793,7 @@ sorted_html(List) -->
 %	Handler for html_receive(head). Unlike  sorted_html//1, it calls
 %	a user hook  html_write:html_head_expansion/2   to  process  the
 %	collected head material into a term suitable for html//1.
-%	
+%
 %	@tbd  This  has  been  added    to   facilate  html_head.pl,  an
 %	experimental  library  for  dealing  with   css  and  javascript
 %	resources. It feels a bit like a hack, but for now I do not know
@@ -829,14 +853,14 @@ post_close(_) -->
 %	Define required newlines before and after   tags.  This table is
 %	rather incomplete. New rules can  be   added  to  this multifile
 %	predicate.
-%	
+%
 %	@param Tag	Name of the tag
 %	@param Open	Tuple M-N, where M is the number of lines before
 %			the tag and N after.
 %	@param Close	Either as Open, or the atom - (minus) to imit the
 %			close-tag or =empty= to indicate the element has
 %			no content model.
-%			
+%
 % 	@tbd	Complete table
 
 :- multifile
@@ -847,7 +871,7 @@ layout(blockquote, 2-1,	1-2).
 layout(pre, 	   2-1,	1-2).
 layout(center,	   2-1,	1-2).
 layout(dl,	   2-1,	1-2).
-layout(ul,	   2-1,	1-2).
+layout(ul,	   1-1,	1-1).
 layout(ol,	   2-1,	1-2).
 layout(form,	   2-1,	1-2).
 layout(frameset,   2-1,	1-2).
@@ -903,7 +927,7 @@ layout(div,	   1-0,	0-1).
 %
 %		* nl(N)
 %		Use at minimum N newlines here.
-%		
+%
 %		* mailbox(Id, Box)
 %		Repositioned tokens (see html_post//2 and
 %		html_receive//2)
@@ -924,7 +948,8 @@ print_html(Out, List) :-
 	;   true
 	),
 	mailman(List),
-	write_html(List, Out).
+	write_html(List, Out),
+	flush_output(Out).
 
 write_html([], _).
 write_html([nl(N)|T], Out) :- !,
@@ -957,7 +982,7 @@ write_nl(N, Out) :-
 %	Determine the content length of  a   token  list  produced using
 %	html//1. Here is an example on  how   this  is used to output an
 %	HTML compatible to HTTP:
-%	
+%
 %	==
 %		phrase(html(DOM), Tokens),
 %		html_print_length(Tokens, Len),
@@ -1044,8 +1069,9 @@ emacs_prolog_colours:goal_colours(html_post(_Id, HTML, _, _),
 					% TBD: Check with do_expand!
 html_colours(Var, classify) :-
 	var(Var), !.
-html_colours(\List, classify) :-
-	is_list(List), !.
+html_colours(\List, built_in-Colours) :-
+	is_list(List), !,
+	list_colours(List, Colours).
 html_colours(\_, built_in-[dcg]) :- !.
 html_colours(_:Term, built_in-[classify,Colours]) :- !,
 	html_colours(Term, Colours).
@@ -1120,7 +1146,7 @@ attr_value_colour(List, classify) :-
 	is_list(List), !.
 attr_value_colour(_, error).
 
-location_id(ID, classify) :- 
+location_id(ID, classify) :-
 	var(ID), !.
 location_id(ID, Class) :-
 	current_predicate(http_dispatch:http_location_by_id/2),
@@ -1158,7 +1184,7 @@ emacs_prolog_colours:identify(http_no_location_for_id(ID), Summary) :-
 
 
 %	prolog:called_by(+Goal, -Called)
-%	
+%
 %	Hook into library(pce_prolog_xref).  Called is a list of callable
 %	or callable+N to indicate (DCG) arglist extension.
 
@@ -1186,7 +1212,7 @@ called_by(Var, _) -->
 	[].
 called_by(\G, M) --> !,
 	(   { is_list(G) }
-	->  []
+	->  called_by(G, M)
 	;   {atom(M)}
 	->  [M:G+2]
 	;   [G+2]
@@ -1201,7 +1227,7 @@ called_by(M:Term, _) --> !,
 	->  called_by(Term, M)
 	;   []
 	).
-called_by(Term, M) --> 
+called_by(Term, M) -->
 	{ compound(Term), !,
 	  Term =.. [_|Args]
 	},
