@@ -117,7 +117,7 @@ instantiation_error()
 { term_t ex = PL_new_term_ref();
 
   PL_unify_term(ex, PL_FUNCTOR, FUNCTOR_error2,
-		      PL_CHARS, "inistantiation_error",
+		      PL_CHARS, "instantiation_error",
 		      PL_VARIABLE);
 
   return PL_raise_exception(ex);
